@@ -26,7 +26,7 @@
 #ifndef _COMPIZ_H
 #define _COMPIZ_H
 
-#define ABIVERSION 20060927
+#define ABIVERSION 20061020
 
 #include <stdio.h>
 #include <sys/time.h>
@@ -457,6 +457,34 @@ getStringOptionNamed (CompOption *option,
 		      char	 *name,
 		      char	 *defaultValue);
 
+char *
+keyBindingToString (CompDisplay    *d,
+		    CompKeyBinding *key);
+
+char *
+buttonBindingToString (CompDisplay       *d,
+		       CompButtonBinding *button);
+
+Bool
+stringToKeyBinding (CompDisplay    *d,
+		    const char     *binding,
+		    CompKeyBinding *key);
+
+Bool
+stringToButtonBinding (CompDisplay	 *d,
+		       const char	 *binding,
+		       CompButtonBinding *button);
+
+char *
+edgeToString (unsigned int edge);
+
+Bool
+stringToColor (const char     *color,
+	       unsigned short *rgba);
+
+char *
+colorToString (unsigned short *rgba);
+
 
 /* display.c */
 
@@ -496,28 +524,30 @@ typedef int CompWatchFdHandle;
 #define COMP_DISPLAY_OPTION_RUN_COMMAND10                 30
 #define COMP_DISPLAY_OPTION_RUN_COMMAND11                 31
 #define COMP_DISPLAY_OPTION_SLOW_ANIMATIONS               32
-#define COMP_DISPLAY_OPTION_LOWER_WINDOW                  33
-#define COMP_DISPLAY_OPTION_UNMAXIMIZE_WINDOW             34
-#define COMP_DISPLAY_OPTION_MINIMIZE_WINDOW               35
-#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW               36
-#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_HORZ          37
-#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_VERT          38
-#define COMP_DISPLAY_OPTION_OPACITY_INCREASE              39
-#define COMP_DISPLAY_OPTION_OPACITY_DECREASE              40
-#define COMP_DISPLAY_OPTION_SCREENSHOT                    41
-#define COMP_DISPLAY_OPTION_RUN_SCREENSHOT                42
-#define COMP_DISPLAY_OPTION_WINDOW_SCREENSHOT             43
-#define COMP_DISPLAY_OPTION_RUN_WINDOW_SCREENSHOT         44
-#define COMP_DISPLAY_OPTION_WINDOW_MENU                   45
-#define COMP_DISPLAY_OPTION_SHOW_DESKTOP                  46
-#define COMP_DISPLAY_OPTION_RAISE_ON_CLICK                47
-#define COMP_DISPLAY_OPTION_AUDIBLE_BELL                  48
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED       49
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_HORZ  50
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_VERT  51
-#define COMP_DISPLAY_OPTION_HIDE_SKIP_TASKBAR_WINDOWS     52
-#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_SHADED          53
-#define COMP_DISPLAY_OPTION_NUM                           54
+#define COMP_DISPLAY_OPTION_RAISE_WINDOW                  33
+#define COMP_DISPLAY_OPTION_LOWER_WINDOW                  34
+#define COMP_DISPLAY_OPTION_UNMAXIMIZE_WINDOW             35
+#define COMP_DISPLAY_OPTION_MINIMIZE_WINDOW               36
+#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW               37
+#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_HORZ          38
+#define COMP_DISPLAY_OPTION_MAXIMIZE_WINDOW_VERT          39
+#define COMP_DISPLAY_OPTION_OPACITY_INCREASE              40
+#define COMP_DISPLAY_OPTION_OPACITY_DECREASE              41
+#define COMP_DISPLAY_OPTION_SCREENSHOT                    42
+#define COMP_DISPLAY_OPTION_RUN_SCREENSHOT                43
+#define COMP_DISPLAY_OPTION_WINDOW_SCREENSHOT             44
+#define COMP_DISPLAY_OPTION_RUN_WINDOW_SCREENSHOT         45
+#define COMP_DISPLAY_OPTION_WINDOW_MENU                   46
+#define COMP_DISPLAY_OPTION_SHOW_DESKTOP                  47
+#define COMP_DISPLAY_OPTION_RAISE_ON_CLICK                48
+#define COMP_DISPLAY_OPTION_AUDIBLE_BELL                  49
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED       50
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_HORZ  51
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_MAXIMIZED_VERT  52
+#define COMP_DISPLAY_OPTION_HIDE_SKIP_TASKBAR_WINDOWS     53
+#define COMP_DISPLAY_OPTION_TOGGLE_WINDOW_SHADED          54
+#define COMP_DISPLAY_OPTION_IGNORE_HINTS_WHEN_MAXIMIZED   55
+#define COMP_DISPLAY_OPTION_NUM                           56
 
 typedef CompOption *(*GetDisplayOptionsProc) (CompDisplay *display,
 					      int	  *count);
@@ -1016,6 +1046,7 @@ struct _CompTexture {
     CompMatrix matrix;
     Bool       oldMipmaps;
     Bool       mipmap;
+    int        refCount;
 };
 
 void
@@ -1026,6 +1057,13 @@ void
 finiTexture (CompScreen  *screen,
 	     CompTexture *texture);
 
+CompTexture *
+createTexture (CompScreen *screen);
+
+void
+destroyTexture (CompScreen  *screen,
+		CompTexture *texture);
+
 Bool
 readImageToTexture (CompScreen   *screen,
 		    CompTexture  *texture,
@@ -1238,6 +1276,8 @@ typedef void (*WindowGrabNotifyProc) (CompWindow   *window,
 
 typedef void (*WindowUngrabNotifyProc) (CompWindow *window);
 
+typedef void (*WindowStateChangeNotifyProc) (CompWindow *window);
+
 #define COMP_SCREEN_DAMAGE_PENDING_MASK (1 << 0)
 #define COMP_SCREEN_DAMAGE_REGION_MASK  (1 << 1)
 #define COMP_SCREEN_DAMAGE_ALL_MASK     (1 << 2)
@@ -1473,6 +1513,8 @@ struct _CompScreen {
     WindowGrabNotifyProc   windowGrabNotify;
     WindowUngrabNotifyProc windowUngrabNotify;
 
+    WindowStateChangeNotifyProc windowStateChangeNotify;
+
     CompPrivate *privates;
 };
 
@@ -1703,11 +1745,14 @@ struct _CompWindow {
     XWindowAttributes attrib;
     int		      serverX;
     int		      serverY;
+    int		      serverWidth;
+    int		      serverHeight;
+    int		      serverBorderWidth;
     Window	      transientFor;
     Window	      clientLeader;
     XSizeHints	      sizeHints;
     Pixmap	      pixmap;
-    CompTexture       texture;
+    CompTexture       *texture;
     CompMatrix        matrix;
     Damage	      damage;
     Bool	      inputHint;
@@ -1796,6 +1841,8 @@ struct _CompWindow {
     int      indexSize;
     int      vCount;
     int      texUnits;
+    int      texCoordSize;
+    int      indexCount;
 
     CompPrivate *privates;
 };
@@ -1927,6 +1974,11 @@ moveWindow (CompWindow *w,
 	    Bool       damage,
 	    Bool       immediate);
 
+void
+configureXWindow (CompWindow	 *w,
+		  unsigned int	 valueMask,
+		  XWindowChanges *xwc);
+
 void
 moveResizeWindow (CompWindow     *w,
 		  XWindowChanges *xwc,
@@ -1997,6 +2049,9 @@ windowGrabNotify (CompWindow   *w,
 void
 windowUngrabNotify (CompWindow *w);
 
+void
+windowStateChangeNotify (CompWindow *w);
+
 void
 moveInputFocusToWindow (CompWindow *w);
 
@@ -2099,14 +2154,19 @@ typedef void (*FiniPluginProc) (CompPlugin *plugin);
 
 typedef enum {
     CompPluginRuleBefore,
-    CompPluginRuleAfter
+    CompPluginRuleAfter,
+    CompPluginRuleRequire
 } CompPluginRule;
 
 typedef struct _CompPluginDep {
     CompPluginRule rule;
-    char	   *plugin;
+    char	   *name;
 } CompPluginDep;
 
+typedef struct _CompPluginFeature {
+    char *name;
+} CompPluginFeature;
+
 typedef struct _CompPluginVTable {
     char *name;
     char *shortDesc;
@@ -2133,6 +2193,9 @@ typedef struct _CompPluginVTable {
 
     CompPluginDep *deps;
     int		  nDeps;
+
+    CompPluginFeature *features;
+    int		      nFeatures;
 } CompPluginVTable;
 
 typedef CompPluginVTable *(*PluginGetInfoProc) (void);
@@ -2187,6 +2250,9 @@ windowFiniPlugins (CompWindow *w);
 CompPlugin *
 findActivePlugin (char *name);
 
+CompPluginFeature *
+findActiveFeature (char *name);
+
 CompPlugin *
 loadPlugin (char *plugin);
 
