@@ -23,10 +23,15 @@
  * Author: David Reveman <davidr@novell.com>
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "../config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+#include <unistd.h>
 
 #include <X11/Xatom.h>
 #include <X11/extensions/shape.h>
@@ -92,6 +97,7 @@ typedef struct _Decoration {
     DecorTexture      *texture;
     CompWindowExtents output;
     CompWindowExtents input;
+    CompWindowExtents maxInput;
     int		      minWidth;
     int		      minHeight;
     Quad	      *quad;
@@ -127,7 +133,8 @@ typedef struct _WindowDecoration {
 #define DECOR_DISPLAY_OPTION_SHADOW_OPACITY  1
 #define DECOR_DISPLAY_OPTION_SHADOW_OFFSET_X 2
 #define DECOR_DISPLAY_OPTION_SHADOW_OFFSET_Y 3
-#define DECOR_DISPLAY_OPTION_NUM             4
+#define DECOR_DISPLAY_OPTION_COMMAND         4
+#define DECOR_DISPLAY_OPTION_NUM             5
 
 static int displayPrivateIndex;
 
@@ -154,6 +161,8 @@ typedef struct _DecorScreen {
 
     WindowMoveNotifyProc   windowMoveNotify;
     WindowResizeNotifyProc windowResizeNotify;
+
+    WindowStateChangeNotifyProc windowStateChangeNotify;
 } DecorScreen;
 
 typedef struct _DecorWindow {
@@ -217,6 +226,29 @@ decorSetDisplayOption (CompDisplay     *display,
     case DECOR_DISPLAY_OPTION_SHADOW_OFFSET_Y:
 	if (compSetIntOption (o, value))
 	    return TRUE;
+	break;
+    case DECOR_DISPLAY_OPTION_COMMAND:
+	if (compSetStringOption (o, value))
+	{
+	    if (display->screens && *o->value.s != '\0')
+	    {
+		DECOR_SCREEN (display->screens);
+
+		/* run decorator command if no decorator is present on
+		   first screen */
+		if (!ds->dmWin)
+		{
+		    if (fork () == 0)
+		    {
+			putenv (display->displayString);
+			execl ("/bin/sh", "/bin/sh", "-c", o->value.s, NULL);
+			exit (0);
+		    }
+		}
+	    }
+
+	    return TRUE;
+	}
     default:
 	break;
     }
@@ -266,6 +298,16 @@ decorDisplayInitOptions (DecorDisplay *dd)
     o->value.i		= DECOR_SHADOW_OFFSET_DEFAULT;
     o->rest.i.min	= DECOR_SHADOW_OFFSET_MIN;
     o->rest.i.max	= DECOR_SHADOW_OFFSET_MAX;
+
+    o = &dd->opt[DECOR_DISPLAY_OPTION_COMMAND];
+    o->name		= "command";
+    o->shortDesc	= N_("Command");
+    o->longDesc		= N_("Decorator command line that is executed if no "
+			     "decorator is already running");
+    o->type		= CompOptionTypeString;
+    o->value.s		= strdup ("");
+    o->rest.s.string	= NULL;
+    o->rest.s.nString	= 0;
 }
 
 static Bool
@@ -415,15 +457,22 @@ decorReleaseTexture (CompScreen   *screen,
   decoration property
   -------------------
 
-  data[0] = pixmap
+  data[0] = version
 
-  data[1] = input left
-  data[2] = input right
-  data[3] = input top
-  data[4] = input bottom
+  data[1] = pixmap
 
-  data[5] = min width
-  data[6] = min height
+  data[2] = input left
+  data[3] = input right
+  data[4] = input top
+  data[5] = input bottom
+
+  data[6] = input left when maximized
+  data[7] = input right when maximized
+  data[8] = input top when maximized
+  data[9] = input bottom when maximized
+
+  data[10] = min width
+  data[11] = min height
 
   flags
 
@@ -431,15 +480,15 @@ decorReleaseTexture (CompScreen   *screen,
   9rd and 10th bit alignment, 11rd and 12th bit clamp,
   13th bit XX, 14th bit XY, 15th bit YX, 16th bit YY.
 
-  data[6 + n * 9 + 1] = flags
-  data[6 + n * 9 + 2] = p1 x
-  data[6 + n * 9 + 3] = p1 y
-  data[6 + n * 9 + 4] = p2 x
-  data[6 + n * 9 + 5] = p2 y
-  data[6 + n * 9 + 6] = widthMax
-  data[6 + n * 9 + 7] = heightMax
-  data[6 + n * 9 + 8] = x0
-  data[6 + n * 9 + 9] = y0
+  data[11 + n * 9 + 1] = flags
+  data[11 + n * 9 + 2] = p1 x
+  data[11 + n * 9 + 3] = p1 y
+  data[11 + n * 9 + 4] = p2 x
+  data[11 + n * 9 + 5] = p2 y
+  data[11 + n * 9 + 6] = widthMax
+  data[11 + n * 9 + 7] = heightMax
+  data[11 + n * 9 + 8] = x0
+  data[11 + n * 9 + 9] = y0
  */
 static Decoration *
 decorCreateDecoration (CompScreen *screen,
@@ -466,12 +515,26 @@ decorCreateDecoration (CompScreen *screen,
     if (result != Success || !n || !data)
 	return NULL;
 
-    if (n < 5 + 9)
+    if (n < 12 + 9)
     {
 	XFree (data);
 	return NULL;
     }
 
+    prop = (long *) data;
+
+    if (*prop != DECOR_INTERFACE_VERSION)
+    {
+	fprintf (stderr, "%s: decoration: property ignored because "
+		 "version is %d and decoration plugin version is %d\n",
+		 programName, (int) *prop, DECOR_INTERFACE_VERSION);
+
+	XFree (data);
+	return NULL;
+    }
+
+    prop++;
+
     decoration = malloc (sizeof (Decoration));
     if (!decoration)
     {
@@ -479,8 +542,6 @@ decorCreateDecoration (CompScreen *screen,
 	return NULL;
     }
 
-    prop = (long *) data;
-
     memcpy (&pixmap, prop++, sizeof (Pixmap));
 
     decoration->texture = decorGetTexture (screen, pixmap);
@@ -496,10 +557,15 @@ decorCreateDecoration (CompScreen *screen,
     decoration->input.top    = *prop++;
     decoration->input.bottom = *prop++;
 
+    decoration->maxInput.left   = *prop++;
+    decoration->maxInput.right  = *prop++;
+    decoration->maxInput.top    = *prop++;
+    decoration->maxInput.bottom = *prop++;
+
     decoration->minWidth  = *prop++;
     decoration->minHeight = *prop++;
 
-    nQuad = (n - 7) / 9;
+    nQuad = (n - 12) / 9;
 
     quad = malloc (sizeof (Quad) * nQuad);
     if (!quad)
@@ -776,7 +842,6 @@ decorWindowUpdate (CompWindow *w,
 {
     WindowDecoration *wd;
     Decoration	     *old, *decor = NULL;
-    int		     dx, dy;
 
     DECOR_SCREEN (w->screen);
     DECOR_WINDOW (w);
@@ -844,27 +909,16 @@ decorWindowUpdate (CompWindow *w,
 	destroyWindowDecoration (w->screen, wd);
     }
 
-    if (decor)
-    {
-	dx = decor->input.left;
-	dy = decor->input.top;
-    }
-    else
-	dx = dy = 0;
-
-    dx -= w->input.left;
-    dy -= w->input.top;
-
-    /* if (dx == 0 && dy == 0) */
-	move = FALSE;
-
     if (decor)
     {
 	dw->wd = createWindowDecoration (decor);
 	if (!dw->wd)
 	    return FALSE;
 
-	setWindowFrameExtents (w, &decor->input, &decor->output);
+	if ((w->state & MAXIMIZE_STATE) == MAXIMIZE_STATE)
+	    setWindowFrameExtents (w, &decor->maxInput, &decor->output);
+	else
+	    setWindowFrameExtents (w, &decor->input, &decor->output);
 
 	if (!move)
 	    damageWindowOutputExtents (w);
@@ -876,9 +930,6 @@ decorWindowUpdate (CompWindow *w,
 	dw->wd = NULL;
     }
 
-    if (move)
-	moveWindow (w, dx, dy, TRUE, TRUE);
-
     return TRUE;
 }
 
@@ -1172,6 +1223,27 @@ decorWindowResizeNotify (CompWindow *w)
     WRAP (ds, w->screen, windowResizeNotify, decorWindowResizeNotify);
 }
 
+static void
+decorWindowStateChangeNotify (CompWindow *w)
+{
+    DECOR_SCREEN (w->screen);
+    DECOR_WINDOW (w);
+
+    if (dw->wd && dw->wd->decor)
+    {
+	Decoration *decor = dw->wd->decor;
+
+	if ((w->state & MAXIMIZE_STATE) == MAXIMIZE_STATE)
+	    setWindowFrameExtents (w, &decor->maxInput, &decor->output);
+	else
+	    setWindowFrameExtents (w, &decor->input, &decor->output);
+    }
+
+    UNWRAP (ds, w->screen, windowStateChangeNotify);
+    (*w->screen->windowStateChangeNotify) (w);
+    WRAP (ds, w->screen, windowStateChangeNotify, decorWindowStateChangeNotify);
+}
+
 static Bool
 decorInitDisplay (CompPlugin  *p,
 		  CompDisplay *d)
@@ -1250,6 +1322,7 @@ decorInitScreen (CompPlugin *p,
     WRAP (ds, s, damageWindowRect, decorDamageWindowRect);
     WRAP (ds, s, windowMoveNotify, decorWindowMoveNotify);
     WRAP (ds, s, windowResizeNotify, decorWindowResizeNotify);
+    WRAP (ds, s, windowStateChangeNotify, decorWindowStateChangeNotify);
 
     s->privates[dd->screenPrivateIndex].ptr = ds;
 
@@ -1274,6 +1347,7 @@ decorFiniScreen (CompPlugin *p,
     UNWRAP (ds, s, damageWindowRect);
     UNWRAP (ds, s, windowMoveNotify);
     UNWRAP (ds, s, windowResizeNotify);
+    UNWRAP (ds, s, windowStateChangeNotify);
 
     free (ds);
 }
@@ -1350,6 +1424,10 @@ CompPluginDep decorDeps[] = {
     { CompPluginRuleBefore, "scale" }
 };
 
+CompPluginFeature decorFeatures[] = {
+    { "decorations" }
+};
+
 static CompPluginVTable decorVTable = {
     "decoration",
     N_("Window Decoration"),
@@ -1368,7 +1446,9 @@ static CompPluginVTable decorVTable = {
     0, /* GetScreenOptions */
     0, /* SetScreenOption */
     decorDeps,
-    sizeof (decorDeps) / sizeof (decorDeps[0])
+    sizeof (decorDeps) / sizeof (decorDeps[0]),
+    decorFeatures,
+    sizeof (decorFeatures) / sizeof (decorFeatures[0])
 };
 
 CompPluginVTable *
