@@ -454,7 +454,7 @@ fadeHandleEvent (CompDisplay *d,
 	{
 	    FADE_SCREEN (w->screen);
 
-	    if (w->texture.pixmap && (fs->wMask & w->type))
+	    if (w->texture->pixmap && (fs->wMask & w->type))
 	    {
 		FADE_WINDOW (w);
 
@@ -481,7 +481,7 @@ fadeHandleEvent (CompDisplay *d,
 
 	    fw->shaded = w->shaded;
 
-	    if (!fw->shaded && w->texture.pixmap && (fs->wMask & w->type))
+	    if (!fw->shaded && w->texture->pixmap && (fs->wMask & w->type))
 	    {
 		w->paint.opacity = 0;
 
@@ -837,7 +837,9 @@ static CompPluginVTable fadeVTable = {
     fadeGetScreenOptions,
     fadeSetScreenOption,
     fadeDeps,
-    sizeof (fadeDeps) / sizeof (fadeDeps[0])
+    sizeof (fadeDeps) / sizeof (fadeDeps[0]),
+    0, /* Features */
+    0  /* nFeatures */
 };
 
 CompPluginVTable *
