@@ -60,7 +60,8 @@
 
 #include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
-#include <gconf-compiz-utils.h>
+
+#define APP_NAME "/apps/compiz"
 
 static FILE *schemaFile;
 
@@ -140,11 +141,7 @@ gconfValueToString (CompDisplay	    *d,
     case CompOptionTypeBool:
 	return g_strdup (value->b ? "true" : "false");
     case CompOptionTypeColor:
-	return g_strdup_printf ("#%.2x%.2x%.2x%.2x",
-				value->c[0] / 256,
-				value->c[1] / 256,
-				value->c[2] / 256,
-				value->c[3] / 256);
+	return colorToString (value->c);
     case CompOptionTypeInt:
 	return g_strdup_printf ("%d", value->i);
     case CompOptionTypeFloat:
@@ -194,7 +191,7 @@ gconfActionValueToString (CompDisplay	  *d,
     {
 	if (value->action.type & CompBindingTypeKey)
 	{
-	    tmp = gconfKeyBindingToString (d, &value->action.key);
+	    tmp = keyBindingToString (d, &value->action.key);
 	    escaped = g_markup_escape_text (tmp, -1);
 	    g_free (tmp);
 	}
@@ -205,7 +202,7 @@ gconfActionValueToString (CompDisplay	  *d,
     {
 	if (value->action.type & CompBindingTypeButton)
 	{
-	    tmp = gconfButtonBindingToString (d, &value->action.button);
+	    tmp = buttonBindingToString (d, &value->action.button);
 	    escaped = g_markup_escape_text (tmp, -1);
 	    g_free (tmp);
 	}
@@ -268,7 +265,7 @@ getLongDesc (CompOption *o, const char *option_name)
 	g_string_append (str, " (");
 	for (i = 0; i < SCREEN_EDGE_NUM; i++)
 	{
-	    edge = gconfEdgeToString (i);
+	    edge = edgeToString (i);
 
 	    if (i > 0)
 		g_string_append (str, ", ");
@@ -296,6 +293,9 @@ getLongDesc (CompOption *o, const char *option_name)
 	int digits = -(logf (o->rest.f.precision) / logf (10));
 	int invprec = 1.0 / o->rest.f.precision;
 
+	if (!invprec)
+	    invprec = 1;
+
 	/* Because we're sort of cheating on the locales, %.*f won't
 	 * work here. We need to localize the format ourselves.
 	 */
@@ -420,7 +420,7 @@ gconfDumpToSchema (CompDisplay *d,
 		{
 		    tmp2 = g_strdup_printf ("%s%s%s", tmp1,
 					    (tmp2) ? "," : "",
-					    gconfEdgeToString (i));
+					    edgeToString (i));
 		    g_free (tmp1);
 
 		    tmp1 = tmp2;
@@ -579,7 +579,7 @@ dumpPluginOptions (CompDisplay *d, CompPlugin *p)
 
 	for (i = 0; i < p->vTable->nDeps; i++)
 	{
-	    value.s = p->vTable->deps[i].plugin;
+	    value.s = p->vTable->deps[i].name;
 	    if (p->vTable->deps[i].rule == CompPluginRuleBefore)
 		g_array_append_val (before, value);
 	    else
@@ -696,21 +696,11 @@ dumpSchema (CompDisplay *d)
 	return 1;
     }
 
-    pluginList = getenv ("COMPIZ_SCHEMA_PLUGINS");
-    if (!pluginList)
-    {
-	fprintf (stderr, "COMPIZ_SCHEMA_PLUGINS must be set\n");
-	return 1;
-    }
-
-    plugins = g_strsplit (pluginList, " ", 0);
-    qsort (plugins, g_strv_length (plugins), sizeof (char *), strcmpref);
-
     schemaFilename = getenv ("COMPIZ_SCHEMA_FILE");
     if (!schemaFilename)
 	schemaFilename = "compiz.schemas.dump";
 
-    schemaFile = fopen (schemaFilename, "w");
+    schemaFile = fopen (schemaFilename, "a");
     if (!schemaFile)
     {
 	fprintf (stderr, "Could not open %s: %s\n", schemaFilename,
@@ -718,30 +708,30 @@ dumpSchema (CompDisplay *d)
 	return 1;
     }
 
-    gconfPrintf (0,
-		 "<!--\n"
-		 "      this file is autogenerated "
-		 "by gconf-dump compiz plugin\n"
-		 "  -->\n\n"
-		 "<gconfschemafile>\n");
-    gconfPrintf (1, "<schemalist>\n\n");
-
     if (getenv ("COMPIZ_SCHEMA_GENERAL"))
 	dumpGeneralOptions (d);
 
-    for (i = 0; plugins[i]; i++)
+    pluginList = getenv ("COMPIZ_SCHEMA_PLUGINS");
+    if (pluginList)
     {
-	CompPlugin *plugin = findActivePlugin (plugins[i]);
-	if (!plugin)
+	plugins = g_strsplit (pluginList, " ", 0);
+	qsort (plugins, g_strv_length (plugins), sizeof (char *), strcmpref);
+
+	for (i = 0; plugins[i]; i++)
 	{
-	    fprintf (stderr, "No such plugin %s\n", plugins[i]);
-	    return 1;
+	    CompPlugin *plugin;
+
+	    plugin = findActivePlugin (plugins[i]);
+	    if (!plugin)
+	    {
+		fprintf (stderr, "No such plugin %s\n", plugins[i]);
+		return 1;
+	    }
+
+	    dumpPluginOptions (d, plugin);
 	}
-	dumpPluginOptions (d, plugin);
     }
 
-    gconfPrintf (1, "</schemalist>\n");
-    gconfPrintf (0, "</gconfschemafile>\n");
     fclose (schemaFile);
 
     return 0;
@@ -843,8 +833,10 @@ CompPluginVTable gconfDumpVTable = {
     0, /* SetDisplayOption */
     0, /* GetScreenOptions */
     0, /* SetScreenOption */
-    0,
-    0
+    0, /* Deps */
+    0, /* nDeps */
+    0, /* Features */
+    0  /* nFeatures */
 };
 
 CompPluginVTable *
