@@ -35,7 +35,7 @@
 #include <glib/gprintf.h>
 #include <gconf/gconf-client.h>
 
-#include <gconf-compiz-utils.h>
+#define APP_NAME "/apps/compiz"
 
 /* From gconf-internal.h. Bleah. */
 int gconf_value_compare (const GConfValue *value_a,
@@ -74,6 +74,20 @@ typedef struct _GConfScreen {
 #define GCONF_SCREEN(s)						           \
     GConfScreen *gs = GET_GCONF_SCREEN (s, GET_GCONF_DISPLAY (s->display))
 
+static int
+strcmpskipifequal (char **ptr,
+		   char *s)
+{
+    int ret, len;
+
+    len = strlen (s);
+    ret = strncmp (*ptr, s, len);
+    if (ret == 0)
+	*ptr = (*ptr) + len;
+
+    return ret;
+}
+
 static GConfValueType
 gconfTypeFromCompType (CompOptionType type)
 {
@@ -121,15 +135,10 @@ gconfSetValue (CompDisplay     *d,
     case CompOptionTypeColor: {
 	gchar *color;
 
-	color = g_strdup_printf ("#%.2x%.2x%.2x%.2x",
-				 value->c[0] / 256,
-				 value->c[1] / 256,
-				 value->c[2] / 256,
-				 value->c[3] / 256);
-
+	color = colorToString (value->c);
 	gconf_value_set_string (gvalue, color);
 
-	g_free (color);
+	free (color);
     } break;
     default:
 	break;
@@ -148,12 +157,12 @@ gconfSetActionValue (CompDisplay     *d,
     if (bindingType == CompBindingTypeKey)
     {
 	if (value->action.type & CompBindingTypeKey)
-	    binding = gconfKeyBindingToString (d, &value->action.key);
+	    binding = keyBindingToString (d, &value->action.key);
     }
     else
     {
 	if (value->action.type & CompBindingTypeButton)
-	    binding = gconfButtonBindingToString (d, &value->action.button);
+	    binding = buttonBindingToString (d, &value->action.button);
     }
 
     if (!binding)
@@ -254,7 +263,7 @@ gconfSetOption (CompDisplay *d,
 	    if (o->value.action.edgeMask & (1 << i))
 	    {
 		gv = gconf_value_new (GCONF_VALUE_STRING);
-		gconf_value_set_string (gv, gconfEdgeToString (i));
+		gconf_value_set_string (gv, edgeToString (i));
 		list = g_slist_append (list, gv);
 	    }
 	}
@@ -349,19 +358,11 @@ gconfGetValue (CompDisplay     *d,
 	     gvalue->type == GCONF_VALUE_STRING)
     {
 	const gchar *color;
-	gint        c[4];
 
 	color = gconf_value_get_string (gvalue);
 
-	if (sscanf (color, "#%2x%2x%2x%2x", &c[0], &c[1], &c[2], &c[3]) == 4)
-	{
-	    value->c[0] = c[0] << 8 | c[0];
-	    value->c[1] = c[1] << 8 | c[1];
-	    value->c[2] = c[2] << 8 | c[2];
-	    value->c[3] = c[3] << 8 | c[3];
-
+	if (stringToColor (color, value->c))
 	    return TRUE;
-	}
     }
 
     return FALSE;
@@ -547,8 +548,8 @@ gconfGetOptionValue (CompDisplay *d,
 		    else
 		    {
 			value.action.type |= CompBindingTypeKey;
-			status = gconfStringToKeyBinding (d, binding,
-							  &value.action.key);
+			status = stringToKeyBinding (d, binding,
+						     &value.action.key);
 		    }
 		}
 		else
@@ -561,8 +562,7 @@ gconfGetOptionValue (CompDisplay *d,
 		    else
 		    {
 			value.action.type |= CompBindingTypeButton;
-			status =
-			    gconfStringToButtonBinding (d, binding,
+			status = stringToButtonBinding (d, binding,
 							&value.action.button);
 		    }
 		}
@@ -598,7 +598,7 @@ gconfGetOptionValue (CompDisplay *d,
 
 			for (i = 0; i < SCREEN_EDGE_NUM; i++)
 			{
-			    if (strcasecmp (edge, gconfEdgeToString (i)) == 0)
+			    if (strcasecmp (edge, edgeToString (i)) == 0)
 				value.action.edgeMask |= 1 << i;
 			}
 
@@ -1065,7 +1065,9 @@ CompPluginVTable gconfVTable = {
     0, /* GetScreenOptions */
     0, /* SetScreenOption */
     gconfDeps,
-    sizeof (gconfDeps) / sizeof (gconfDeps[0])
+    sizeof (gconfDeps) / sizeof (gconfDeps[0]),
+    0, /* Features */
+    0  /* nFeatures */
 };
 
 CompPluginVTable *
