@@ -1580,7 +1580,9 @@ CompPluginVTable scaleVTable = {
     scaleGetScreenOptions,
     scaleSetScreenOption,
     0, /* Deps */
-    0  /* nDeps */
+    0, /* nDeps */
+    0, /* Features */
+    0  /* nFeatures */
 };
 
 CompPluginVTable *
