@@ -115,6 +115,9 @@ int pointerY     = 0;
 #define MAXIMIZE_WINDOW_KEY_DEFAULT       "F10"
 #define MAXIMIZE_WINDOW_MODIFIERS_DEFAULT CompAltMask
 
+#define RAISE_WINDOW_BUTTON_DEFAULT    6
+#define RAISE_WINDOW_MODIFIERS_DEFAULT ControlMask
+
 #define LOWER_WINDOW_BUTTON_DEFAULT    6
 #define LOWER_WINDOW_MODIFIERS_DEFAULT CompAltMask
 
@@ -148,6 +151,8 @@ int pointerY     = 0;
 #define TOGGLE_WINDOW_SHADING_KEY_DEFAULT       "s"
 #define TOGGLE_WINDOW_SHADING_MODIFIERS_DEFAULT (CompAltMask | ControlMask)
 
+#define IGNORE_HINTS_WHEN_MAXIMIZED_DEFAULT TRUE
+
 #define NUM_OPTIONS(d) (sizeof ((d)->opt) / sizeof (CompOption))
 
 static char *textureFilter[] = { N_("Fast"), N_("Good"), N_("Best") };
@@ -398,6 +403,25 @@ toggleSlowAnimations (CompDisplay     *d,
     return TRUE;
 }
 
+static Bool
+raise (CompDisplay     *d,
+       CompAction      *action,
+       CompActionState state,
+       CompOption      *option,
+       int	       nOption)
+{
+    CompWindow *w;
+    Window     xid;
+
+    xid = getIntOptionNamed (option, nOption, "window", 0);
+
+    w = findTopLevelWindowAtDisplay (d, xid);
+    if (w)
+	raiseWindow (w);
+
+    return TRUE;
+}
+
 static Bool
 lower (CompDisplay     *d,
        CompAction      *action,
@@ -948,6 +972,21 @@ compDisplayInitOptions (CompDisplay *display,
 	XKeysymToKeycode (display->display,
 			  XStringToKeysym (SLOW_ANIMATIONS_KEY_DEFAULT));
 
+    o = &display->opt[COMP_DISPLAY_OPTION_RAISE_WINDOW];
+    o->name			     = "raise_window";
+    o->shortDesc		     = N_("Raise Window");
+    o->longDesc			     = N_("Raise window above other windows");
+    o->type			     = CompOptionTypeAction;
+    o->value.action.initiate	     = raise;
+    o->value.action.terminate        = 0;
+    o->value.action.bell	     = FALSE;
+    o->value.action.edgeMask	     = 0;
+    o->value.action.state	     = CompActionStateInitKey;
+    o->value.action.state	    |= CompActionStateInitButton;
+    o->value.action.type	     = CompBindingTypeButton;
+    o->value.action.button.modifiers = RAISE_WINDOW_MODIFIERS_DEFAULT;
+    o->value.action.button.button    = RAISE_WINDOW_BUTTON_DEFAULT;
+
     o = &display->opt[COMP_DISPLAY_OPTION_LOWER_WINDOW];
     o->name			     = "lower_window";
     o->shortDesc		     = N_("Lower Window");
@@ -1145,6 +1184,14 @@ compDisplayInitOptions (CompDisplay *display,
     o->value.action.key.keycode   =
 	XKeysymToKeycode (display->display,
 			  XStringToKeysym (TOGGLE_WINDOW_SHADING_KEY_DEFAULT));
+
+    o = &display->opt[COMP_DISPLAY_OPTION_IGNORE_HINTS_WHEN_MAXIMIZED];
+    o->name	 = "ignore_hints_when_maximized";
+    o->shortDesc = N_("Ignore Hints When Maximized");
+    o->longDesc	 = N_("Ignore size increment and aspect hints when window is "
+	"maximized");
+    o->type	 = CompOptionTypeBool;
+    o->value.b	 = IGNORE_HINTS_WHEN_MAXIMIZED_DEFAULT;
 }
 
 CompOption *
@@ -1206,6 +1253,7 @@ setDisplayOption (CompDisplay     *display,
     case COMP_DISPLAY_OPTION_AUTORAISE:
     case COMP_DISPLAY_OPTION_RAISE_ON_CLICK:
     case COMP_DISPLAY_OPTION_HIDE_SKIP_TASKBAR_WINDOWS:
+    case COMP_DISPLAY_OPTION_IGNORE_HINTS_WHEN_MAXIMIZED:
 	if (compSetBoolOption (o, value))
 	    return TRUE;
 	break;
@@ -1252,6 +1300,7 @@ setDisplayOption (CompDisplay     *display,
     case COMP_DISPLAY_OPTION_RUN_COMMAND10:
     case COMP_DISPLAY_OPTION_RUN_COMMAND11:
     case COMP_DISPLAY_OPTION_SLOW_ANIMATIONS:
+    case COMP_DISPLAY_OPTION_RAISE_WINDOW:
     case COMP_DISPLAY_OPTION_LOWER_WINDOW:
     case COMP_DISPLAY_OPTION_OPACITY_INCREASE:
     case COMP_DISPLAY_OPTION_OPACITY_DECREASE:
@@ -2351,6 +2400,7 @@ addScreenActions (CompDisplay *d, CompScreen *s)
     addScreenAction (s, &d->opt[COMP_DISPLAY_OPTION_RUN_COMMAND11].value.action);
     addScreenAction (s,
 		     &d->opt[COMP_DISPLAY_OPTION_SLOW_ANIMATIONS].value.action);
+    addScreenAction (s, &d->opt[COMP_DISPLAY_OPTION_RAISE_WINDOW].value.action);
     addScreenAction (s, &d->opt[COMP_DISPLAY_OPTION_LOWER_WINDOW].value.action);
     addScreenAction (s,
 		     &d->opt[COMP_DISPLAY_OPTION_OPACITY_INCREASE].value.action);
