@@ -110,12 +110,12 @@ recalcNormalHints (CompWindow *window)
     int maxSize;
 
     maxSize = window->screen->maxTextureSize;
-    maxSize -= window->attrib.border_width * 2;
+    maxSize -= window->serverBorderWidth * 2;
 
     window->sizeHints.x      = window->attrib.x;
     window->sizeHints.y      = window->attrib.y;
-    window->sizeHints.width  = window->attrib.width;
-    window->sizeHints.height = window->attrib.height;
+    window->sizeHints.width  = window->serverWidth;
+    window->sizeHints.height = window->serverHeight;
 
     if (window->sizeHints.flags & PMinSize)
     {
@@ -570,23 +570,37 @@ recalcWindowActions (CompWindow *w)
     switch (w->type) {
     case CompWindowTypeFullscreenMask:
     case CompWindowTypeNormalMask:
-    case CompWindowTypeUtilMask:
-    case CompWindowTypeToolbarMask:
-    case CompWindowTypeDialogMask:
-    case CompWindowTypeModalDialogMask:
-    case CompWindowTypeMenuMask:
 	actions =
-		CompWindowActionMaximizeHorzMask |
-		CompWindowActionMaximizeVertMask |
-		CompWindowActionFullscreenMask   |
-		CompWindowActionMoveMask         |
-		CompWindowActionResizeMask       |
-		CompWindowActionStickMask        |
-		CompWindowActionMinimizeMask     |
-		CompWindowActionCloseMask;
-    if (w->input.top)
+	    CompWindowActionMaximizeHorzMask |
+	    CompWindowActionMaximizeVertMask |
+	    CompWindowActionFullscreenMask   |
+	    CompWindowActionMoveMask         |
+	    CompWindowActionResizeMask       |
+	    CompWindowActionStickMask        |
+	    CompWindowActionMinimizeMask     |
+	    CompWindowActionCloseMask;
+
+	if (w->input.top)
 	    actions |= CompWindowActionShadeMask;
 	break;
+    case CompWindowTypeUtilMask:
+    case CompWindowTypeToolbarMask:
+	actions =
+	    CompWindowActionMoveMask   |
+	    CompWindowActionResizeMask |
+	    CompWindowActionStickMask  |
+	    CompWindowActionCloseMask;
+
+	if (w->input.top)
+	    actions |= CompWindowActionShadeMask;
+	break;
+    case CompWindowTypeDialogMask:
+    case CompWindowTypeModalDialogMask:
+	actions =
+	    CompWindowActionMoveMask   |
+	    CompWindowActionResizeMask |
+	    CompWindowActionStickMask  |
+	    CompWindowActionCloseMask;
     default:
 	break;
     }
@@ -995,7 +1009,7 @@ setWindowFrameExtents (CompWindow	 *w,
 static void
 setWindowMatrix (CompWindow *w)
 {
-    w->matrix = w->texture.matrix;
+    w->matrix = w->texture->matrix;
     w->matrix.x0 -= (w->attrib.x * w->matrix.xx);
     w->matrix.y0 -= (w->attrib.y * w->matrix.yy);
 }
@@ -1016,7 +1030,7 @@ bindWindow (CompWindow *w)
 	if (attr.map_state != IsViewable)
 	{
 	    XUngrabServer (w->screen->display->display);
-	    finiTexture (w->screen, &w->texture);
+	    finiTexture (w->screen, w->texture);
 	    w->mapNum = 0;
 	    return;
 	}
@@ -1027,7 +1041,7 @@ bindWindow (CompWindow *w)
 	XUngrabServer (w->screen->display->display);
     }
 
-    if (!bindPixmapToTexture (w->screen, &w->texture, w->pixmap,
+    if (!bindPixmapToTexture (w->screen, w->texture, w->pixmap,
 			      w->width, w->height,
 			      w->attrib.depth))
     {
@@ -1043,7 +1057,16 @@ releaseWindow (CompWindow *w)
 {
     if (w->pixmap)
     {
-	releasePixmapFromTexture (w->screen, &w->texture);
+	CompTexture *texture;
+
+	texture = createTexture (w->screen);
+	if (texture)
+	{
+	    destroyTexture (w->screen, w->texture);
+
+	    w->texture = texture;
+	}
+
 	XFreePixmap (w->screen->display->display, w->pixmap);
 
 	w->pixmap = None;
@@ -1061,8 +1084,7 @@ freeWindow (CompWindow *w)
     if (w->syncWaitHandle)
 	compRemoveTimeout (w->syncWaitHandle);
 
-    if (w->texture.name)
-	finiTexture (w->screen, &w->texture);
+    destroyTexture (w->screen, w->texture);
 
     if (w->frame)
 	XDestroyWindow (w->screen->display->display, w->frame);
@@ -1496,7 +1518,8 @@ addWindow (CompScreen *screen,
     w->shaded		 = FALSE;
     w->hidden		 = FALSE;
 
-    w->initialViewport = screen->x;
+    w->initialViewportX = screen->x;
+    w->initialViewportY = screen->y;
 
     w->pendingUnmaps = 0;
 
@@ -1504,7 +1527,12 @@ addWindow (CompScreen *screen,
     w->resName   = NULL;
     w->resClass  = NULL;
 
-    initTexture (screen, &w->texture);
+    w->texture = createTexture (screen);
+    if (!w->texture)
+    {
+	free (w);
+	return;
+    }
 
     w->screen     = screen;
     w->pixmap     = None;
@@ -1522,11 +1550,13 @@ addWindow (CompScreen *screen,
     w->sizeDamage  = 0;
     w->nDamage	   = 0;
 
-    w->vertices   = 0;
-    w->vertexSize = 0;
-    w->indices    = 0;
-    w->indexSize  = 0;
-    w->vCount     = 0;
+    w->vertices     = 0;
+    w->vertexSize   = 0;
+    w->indices      = 0;
+    w->indexSize    = 0;
+    w->vCount	    = 0;
+    w->indexCount   = 0;
+    w->texCoordSize = 2;
 
     w->struts = 0;
 
@@ -1570,6 +1600,7 @@ addWindow (CompScreen *screen,
 	w->privates = malloc (screen->windowPrivateLen * sizeof (CompPrivate));
 	if (!w->privates)
 	{
+	    destroyTexture (screen, w->texture);
 	    free (w);
 	    return;
 	}
@@ -1598,6 +1629,10 @@ addWindow (CompScreen *screen,
     if (!XGetWindowAttributes (screen->display->display, id, &w->attrib))
 	setDefaultWindowAttributes (&w->attrib);
 
+    w->serverWidth	 = w->attrib.width;
+    w->serverHeight	 = w->attrib.height;
+    w->serverBorderWidth = w->attrib.border_width;
+
     w->width  = w->attrib.width  + w->attrib.border_width * 2;
     w->height = w->attrib.height + w->attrib.border_width * 2;
 
@@ -1662,8 +1697,6 @@ addWindow (CompScreen *screen,
 
 	XUnionRegion (&rect, w->region, w->region);
 
-	initTexture (screen, &w->texture);
-
 	w->damage = XDamageCreate (screen->display->display, id,
 				   XDamageReportRawRectangles);
 
@@ -1680,6 +1713,7 @@ addWindow (CompScreen *screen,
 
     w->invisible = TRUE;
 
+    w->wmType    = getWindowType (w->screen->display, w->id);
     w->protocols = getProtocols (w->screen->display, w->id);
 
     if (!w->attrib.override_redirect)
@@ -1699,8 +1733,6 @@ addWindow (CompScreen *screen,
 	if (!w->clientLeader)
 	    w->startupId = getStartupId (w);
 
-	w->wmType = getWindowType (w->screen->display, w->id);
-	
 	recalcWindowType (w);
 
 	getMwmHints (w->screen->display, w->id, &w->mwmFunc, &w->mwmDecor);
@@ -1734,6 +1766,10 @@ addWindow (CompScreen *screen,
 		w->paint.saturation = w->saturation;
 	}
     }
+    else
+    {
+	recalcWindowType (w);
+    }
 
     if (w->attrib.map_state == IsViewable)
     {
@@ -1813,61 +1849,57 @@ destroyWindow (CompWindow *w)
     }
 }
 
-static Bool
-configurePredicate (Display  *display,
-		    XEvent   *event,
-		    XPointer arg)
-{
-    if (event->type == ConfigureNotify)
-    {
-	XConfigureEvent *xev = (XConfigureEvent *) arg;
-
-	if (event->xconfigure.window == xev->window)
-	{
-	    xev->x      = event->xconfigure.x;
-	    xev->y      = event->xconfigure.y;
-	    xev->width  = event->xconfigure.width;
-	    xev->height = event->xconfigure.height;
-
-	    xev->border_width = event->xconfigure.border_width;
-
-	    xev->above	           = event->xconfigure.above;
-	    xev->override_redirect = event->xconfigure.override_redirect;
-	}
-    }
-
-    return FALSE;
-}
-
 void
 sendConfigureNotify (CompWindow *w)
 {
     XConfigureEvent xev;
-    XEvent	    event;
 
     xev.type	     = ConfigureNotify;
     xev.event	     = w->id;
     xev.window	     = w->id;
-    xev.x	     = w->attrib.x;
-    xev.y	     = w->attrib.y;
-    xev.width	     = w->attrib.width;
-    xev.height	     = w->attrib.height;
-    xev.border_width = w->attrib.border_width;
 
-    xev.above	          = (w->prev) ? w->prev->id : None;
-    xev.override_redirect = w->attrib.override_redirect;
+    /* normally we should never send configure notify events to override
+       redirect windows but if they support the _NET_WM_SYNC_REQUEST
+       protocol we need to do this when the window is mapped. however the
+       only way we can make sure that the attributes we send are correct
+       and is to grab the server. */
+    if (w->attrib.override_redirect)
+    {
+	XWindowAttributes attrib;
 
-    /* If we have pending ConfigureNotifies from any previous
-       ConfigureRequests we like those changes to be in this synthetic
-       ConfigureNotify */
-    XSync (w->screen->display->display, FALSE);
+	XGrabServer (w->screen->display->display);
 
-    while (XCheckIfEvent (w->screen->display->display, &event,
-			  configurePredicate,
-			  (XPointer) &xev));
+	if (XGetWindowAttributes (w->screen->display->display, w->id, &attrib))
+	{
+	    xev.x	     = attrib.x;
+	    xev.y	     = attrib.y;
+	    xev.width	     = attrib.width;
+	    xev.height	     = attrib.height;
+	    xev.border_width = attrib.border_width;
 
-    XSendEvent (w->screen->display->display, w->id, FALSE,
-		StructureNotifyMask, (XEvent *) &xev);
+	    xev.above	      = (w->prev) ? w->prev->id : None;
+	    xev.override_redirect = TRUE;
+
+	    XSendEvent (w->screen->display->display, w->id, FALSE,
+			StructureNotifyMask, (XEvent *) &xev);
+	}
+
+	XUngrabServer (w->screen->display->display);
+    }
+    else
+    {
+	xev.x	     = w->attrib.x;
+	xev.y	     = w->attrib.y;
+	xev.width	     = w->serverWidth;
+	xev.height	     = w->serverHeight;
+	xev.border_width = w->serverBorderWidth;
+
+	xev.above	          = (w->prev) ? w->prev->id : None;
+	xev.override_redirect = w->attrib.override_redirect;
+
+	XSendEvent (w->screen->display->display, w->id, FALSE,
+		    StructureNotifyMask, (XEvent *) &xev);
+    }
 }
 
 void
@@ -2208,9 +2240,9 @@ sendSyncRequest (CompWindow *w)
     w->syncWait	       = TRUE;
     w->syncX	       = w->serverX;
     w->syncY	       = w->serverY;
-    w->syncWidth       = w->attrib.width;
-    w->syncHeight      = w->attrib.height;
-    w->syncBorderWidth = w->attrib.border_width;
+    w->syncWidth       = w->serverWidth;
+    w->syncHeight      = w->serverHeight;
+    w->syncBorderWidth = w->serverBorderWidth;
 
     if (!w->syncWaitHandle)
 	w->syncWaitHandle = compAddTimeout (1000, syncWaitTimeout, w);
@@ -2230,6 +2262,13 @@ configureWindow (CompWindow	 *w,
     }
     else
     {
+	if (ce->override_redirect)
+	{
+	    w->serverWidth       = ce->width;
+	    w->serverHeight      = ce->height;
+	    w->serverBorderWidth = ce->border_width;
+	}
+
 	resizeWindow (w, ce->x, ce->y, ce->width, ce->height,
 		      ce->border_width);
     }
@@ -2286,17 +2325,12 @@ moveWindow (CompWindow *w,
 void
 syncWindowPosition (CompWindow *w)
 {
-    if (w->attrib.x != w->serverX || w->attrib.y != w->serverY)
-    {
-	XMoveWindow (w->screen->display->display, w->id,
-		     w->attrib.x,
-		     w->attrib.y);
+    XMoveWindow (w->screen->display->display, w->id, w->attrib.x, w->attrib.y);
 
-	if (w->frame)
-	    XMoveWindow (w->screen->display->display, w->frame,
-			 w->attrib.x - w->input.left,
-			 w->attrib.y - w->input.top);
-    }
+    if (w->frame)
+	XMoveWindow (w->screen->display->display, w->frame,
+		     w->attrib.x - w->input.left,
+		     w->attrib.y - w->input.top);
 }
 
 void
@@ -2366,6 +2400,11 @@ windowUngrabNotify (CompWindow *w)
 {
 }
 
+void
+windowStateChangeNotify (CompWindow *w)
+{
+}
+
 static Bool
 isGroupTransient (CompWindow *w,
 		  Window     clientLeader)
@@ -2464,7 +2503,28 @@ moveInputFocusToWindow (CompWindow *w)
     }
     else
     {
-	XSetInputFocus (d->display, w->id, RevertToPointerRoot, CurrentTime);
+	if (w->inputHint)
+	{
+	    XSetInputFocus (d->display, w->id, RevertToPointerRoot,
+			    CurrentTime);
+	}
+	else if (w->protocols & CompWindowProtocolTakeFocusMask)
+	{
+	    XEvent ev;
+
+	    ev.type		    = ClientMessage;
+	    ev.xclient.window	    = w->id;
+	    ev.xclient.message_type = d->wmProtocolsAtom;
+	    ev.xclient.format	    = 32;
+	    ev.xclient.data.l[0]    = d->wmTakeFocusAtom;
+	    ev.xclient.data.l[1]    =
+		getCurrentTimeFromDisplay (d);
+	    ev.xclient.data.l[2]    = 0;
+	    ev.xclient.data.l[3]    = 0;
+	    ev.xclient.data.l[4]    = 0;
+
+	    XSendEvent (d->display, w->id, FALSE, NoEventMask, &ev);
+	}
     }
 }
 
@@ -2723,13 +2783,13 @@ saveWindowGeometry (CompWindow *w,
 	w->saveWc.y = w->attrib.y;
 
     if (m & CWWidth)
-	w->saveWc.width = w->attrib.width;
+	w->saveWc.width = w->serverWidth;
 
     if (m & CWHeight)
-	w->saveWc.height = w->attrib.height;
+	w->saveWc.height = w->serverHeight;
 
     if (m & CWBorderWidth)
-	w->saveWc.border_width = w->attrib.border_width;
+	w->saveWc.border_width = w->serverBorderWidth;
 
     w->saveMask |= m;
 }
@@ -2755,7 +2815,7 @@ restoreWindowGeometry (CompWindow     *w,
 	   the same as the current width then make it a little be smaller so
 	   the user can see that it changed and it also makes sure that
 	   windowResizeNotify is called and plugins are notified. */
-	if (xwc->width == w->attrib.width)
+	if (xwc->width == w->serverWidth)
 	{
 	    xwc->width -= 10;
 	    if (m & CWX)
@@ -2769,7 +2829,7 @@ restoreWindowGeometry (CompWindow     *w,
 
 	/* As above, if the saved height is the same as the current height
 	   then make it a little be smaller. */
-	if (xwc->height == w->attrib.height)
+	if (xwc->height == w->serverHeight)
 	{
 	    xwc->height -= 10;
 	    if (m & CWY)
@@ -2785,12 +2845,20 @@ restoreWindowGeometry (CompWindow     *w,
     return m;
 }
 
-static void
-configureXWindow (Display	 *dpy,
-		  CompWindow	 *w,
+void
+configureXWindow (CompWindow	 *w,
 		  unsigned int	 valueMask,
 		  XWindowChanges *xwc)
 {
+    if (valueMask & CWWidth)
+	w->serverWidth = xwc->width;
+
+    if (valueMask & CWHeight)
+	w->serverHeight	= xwc->height;
+
+    if (valueMask & CWBorderWidth)
+	w->serverBorderWidth = xwc->border_width;
+
     XConfigureWindow (w->screen->display->display, w->id,
 		      valueMask, xwc);
 
@@ -2824,8 +2892,7 @@ stackTransients (CompWindow	*w,
 		return FALSE;
 
 	    if (t->mapNum)
-		configureXWindow (w->screen->display->display, t,
-				  CWSibling | CWStackMode, xwc);
+		configureXWindow (t, CWSibling | CWStackMode, xwc);
 	}
     }
 
@@ -2850,7 +2917,7 @@ stackAncestors (CompWindow     *w,
 		return;
 
 	    if (ancestor->mapNum)
-		configureXWindow (w->screen->display->display, ancestor,
+		configureXWindow (ancestor,
 				  CWSibling | CWStackMode,
 				  xwc);
 
@@ -2877,7 +2944,7 @@ stackAncestors (CompWindow     *w,
 		    continue;
 
 		if (a->mapNum)
-		    configureXWindow (w->screen->display->display, a,
+		    configureXWindow (a,
 				      CWSibling | CWStackMode,
 				      xwc);
 	    }
@@ -2895,26 +2962,21 @@ addWindowSizeChanges (CompWindow     *w,
 		      int	     oldBorderWidth)
 {
     int mask = 0;
-    int x;
-    int head;
+    int x, y;
+    int vx, vy;
 
-    x = (defaultViewportForWindow (w) - w->screen->x) * w->screen->width;
+    defaultViewportForWindow (w, &vx, &vy);
 
-    // Some options should be added there for Complete Maximising and Complete Fullscreening
-    head = screenGetHeadForWindow ( w );
+    x = (vx - w->screen->x) * w->screen->width;
+    y = (vy - w->screen->y) * w->screen->height;
 
     if (w->type & CompWindowTypeFullscreenMask)
     {
 	saveWindowGeometry (w, CWX | CWY | CWWidth | CWHeight | CWBorderWidth);
-/*
+
 	xwc->width	  = w->screen->width;
 	xwc->height	  = w->screen->height;
 	xwc->border_width = 0;
-*/
-
-	xwc->width	  = screenGetHeadWidth ( w->screen, head );
-	xwc->height	  = screenGetHeadHeight ( w->screen, head );
-	xwc->border_width = 0;
 
 	mask |= CWWidth | CWHeight | CWBorderWidth;
     }
@@ -2926,7 +2988,7 @@ addWindowSizeChanges (CompWindow     *w,
 	{
 	    saveWindowGeometry (w, CWY | CWHeight);
 
-        xwc->height = screenGetHeadAreaHeight ( w->screen, head ) - w->input.top -
+	    xwc->height = w->screen->workArea.height - w->input.top -
 		w->input.bottom - oldBorderWidth * 2;
 
 	    mask |= CWHeight;
@@ -2940,7 +3002,7 @@ addWindowSizeChanges (CompWindow     *w,
 	{
 	    saveWindowGeometry (w, CWX | CWWidth);
 
-	    xwc->width = screenGetHeadAreaWidth ( w->screen, head ) - w->input.left -
+	    xwc->width = w->screen->workArea.width - w->input.left -
 		w->input.right - oldBorderWidth * 2;
 
 	    mask |= CWWidth;
@@ -2951,16 +3013,16 @@ addWindowSizeChanges (CompWindow     *w,
 	}
 
 	/* constrain window width if greater than maximum width */
-	if (!(mask & CWWidth) && w->attrib.width > w->sizeHints.max_width)
+	if (!(mask & CWWidth) && w->serverWidth > w->sizeHints.max_width)
 	{
-	    xwc->width = w->attrib.width;
+	    xwc->width = w->sizeHints.max_width;
 	    mask |= CWWidth;
 	}
 
 	/* constrain window height if greater than maximum height */
-	if (!(mask & CWHeight) && w->attrib.height > w->sizeHints.max_height)
+	if (!(mask & CWHeight) && w->serverHeight > w->sizeHints.max_height)
 	{
-	    xwc->height = w->attrib.height;
+	    xwc->height = w->sizeHints.max_height;
 	    mask |= CWHeight;
 	}
     }
@@ -2969,10 +3031,8 @@ addWindowSizeChanges (CompWindow     *w,
     {
 	if (w->type & CompWindowTypeFullscreenMask)
 	{
-//      xwc->x = x;
-//      xwc->y = 0;
-	    xwc->x = screenGetHeadX ( w->screen, head );
-	    xwc->y = screenGetHeadY ( w->screen, head );
+	    xwc->x = x;
+	    xwc->y = y;
 
 	    mask |= CWX | CWY;
 	}
@@ -2996,17 +3056,16 @@ addWindowSizeChanges (CompWindow     *w,
 
 	    if (w->state & CompWindowStateMaximizedVertMask)
 	    {
-//		if (w->attrib.y < w->screen->workArea.y + w->input.top)
-        if (oldY < screenGetHeadAreaY ( w->screen, head ) + w->input.top)
+		if (oldY < y + w->screen->workArea.y + w->input.top)
 		{
-		    xwc->y = screenGetHeadAreaY ( w->screen, head ) + w->input.top;
+		    xwc->y = y + w->screen->workArea.y + w->input.top;
 		    mask |= CWY;
 		}
 		else
 		{
 		    height = xwc->height + oldBorderWidth * 2;
 
-            max = screenGetHeadAreaY ( w->screen, head ) + screenGetHeadAreaHeight ( w->screen, head );
+		    max = y + w->screen->workArea.y + w->screen->workArea.height;
 		    if (oldY + oldHeight + w->input.bottom > max)
 		    {
 			xwc->y = max - height - w->input.bottom;
@@ -3014,8 +3073,8 @@ addWindowSizeChanges (CompWindow     *w,
 		    }
 		    else if (oldY + height + w->input.bottom > max)
 		    {
-			xwc->y = screenGetHeadAreaY(w->screen,head) +
-			    (screenGetHeadAreaHeight(w->screen,head) - w->input.top -
+			xwc->y = w->screen->workArea.y +
+			    (w->screen->workArea.height - w->input.top -
 			     height - w->input.bottom) / 2 + w->input.top;
 			mask |= CWY;
 		    }
@@ -3024,16 +3083,16 @@ addWindowSizeChanges (CompWindow     *w,
 
 	    if (w->state & CompWindowStateMaximizedHorzMask)
 	    {
-        if (oldX < x + screenGetHeadAreaX ( w->screen, head ) + w->input.left)
+		if (oldX < x + w->screen->workArea.x + w->input.left)
 		{
-		    xwc->x = x + screenGetHeadAreaX ( w->screen, head ) + w->input.left;
+		    xwc->x = x + w->screen->workArea.x + w->input.left;
 		    mask |= CWX;
 		}
 		else
 		{
 		    width = xwc->width + oldBorderWidth * 2;
 
-            max = x + screenGetHeadAreaX ( w->screen, head ) + screenGetHeadAreaWidth ( w->screen, head );
+		    max = x + w->screen->workArea.x + w->screen->workArea.width;
 		    if (oldX + oldWidth + w->input.right > max)
 		    {
 			xwc->x = max - width - w->input.right;
@@ -3041,8 +3100,8 @@ addWindowSizeChanges (CompWindow     *w,
 		    }
 		    else if (oldX + width + w->input.right > max)
 		    {
-			xwc->x = screenGetHeadAreaX(w->screen,head) +
-			    (screenGetHeadAreaWidth(w->screen,head) - w->input.left -
+			xwc->x = w->screen->workArea.x +
+			    (w->screen->workArea.width - w->input.left -
 			     width - w->input.right) / 2 + w->input.left;
 			mask |= CWX;
 		    }
@@ -3060,6 +3119,8 @@ moveResizeWindow (CompWindow     *w,
 		  unsigned int   xwcm,
 		  int            gravity)
 {
+    Bool placed = xwcm & (CWX | CWY);
+
     xwcm &= (CWX | CWY | CWWidth | CWHeight | CWBorderWidth);
 
     if (gravity == 0)
@@ -3070,9 +3131,9 @@ moveResizeWindow (CompWindow     *w,
     if (!(xwcm & CWY))
 	xwc->y = w->attrib.y;
     if (!(xwcm & CWWidth))
-	xwc->width = w->attrib.width;
+	xwc->width = w->serverWidth;
     if (!(xwcm & CWHeight))
-	xwc->height = w->attrib.height;
+	xwc->height = w->serverHeight;
 
     if (xwcm & (CWWidth | CWHeight))
     {
@@ -3098,7 +3159,7 @@ moveResizeWindow (CompWindow     *w,
 	case CenterGravity:
 	case SouthGravity:
 	    if (!(xwcm & CWX))
-		xwc->x += (w->attrib.width - xwc->width) / 2;
+		xwc->x += (w->serverWidth - xwc->width) / 2;
 	    break;
 
 	case NorthEastGravity:
@@ -3107,7 +3168,7 @@ moveResizeWindow (CompWindow     *w,
 	    if (xwcm & CWX)
 		xwc->x -= w->input.right;
 	    else
-		xwc->x += w->attrib.width - xwc->width;
+		xwc->x += w->serverWidth - xwc->width;
 	    break;
 
 	case StaticGravity:
@@ -3125,7 +3186,7 @@ moveResizeWindow (CompWindow     *w,
 	case CenterGravity:
 	case EastGravity:
 	    if (!(xwcm & CWY))
-		xwc->y += (w->attrib.height - xwc->height) / 2;
+		xwc->y += (w->serverHeight - xwc->height) / 2;
 	    break;
 
 	case SouthWestGravity:
@@ -3134,7 +3195,7 @@ moveResizeWindow (CompWindow     *w,
 	    if (xwcm & CWY)
 		xwc->y -= w->input.bottom;
 	    else
-		xwc->y += w->attrib.height - xwc->height;
+		xwc->y += w->serverHeight - xwc->height;
 	    break;
 
 	case StaticGravity:
@@ -3164,30 +3225,34 @@ moveResizeWindow (CompWindow     *w,
 
 	if (xwcm & CWX)
 	{
-	    int min, max, viewport, x;
-
-	    viewport = defaultViewportForWindow ( w );
+	    int min, max;
 
 	    min = w->screen->workArea.x + w->input.left;
-	    max = w->screen->workArea.x + w->screen->width * w->screen->size;
-	    
-	    x = w->attrib.x + viewport * w->screen->width + xwc->x;
+	    max = w->screen->workArea.x + w->screen->workArea.width;
 
-	    if (x < min)
-		x = min;
-	    else if (x > max)
-		x = max;
-	    
-	    xwc->x = x - w->attrib.x - viewport * w->screen->width;
+	    if (xwc->x < min)
+		xwc->x = min;
+	    else if (xwc->x > max)
+		xwc->x = max;
 	}
     }
 
     if (xwcm & CWBorderWidth)
     {
-	if (xwc->border_width == w->attrib.border_width)
+	if (xwc->border_width == w->serverBorderWidth)
 	    xwcm &= ~CWBorderWidth;
     }
 
+    /* when horizontally maximized only allow width changes added by
+       addWindowSizeChanges */
+    if (w->state & CompWindowStateMaximizedHorzMask)
+	xwcm &= ~CWWidth;
+
+    /* when vertically maximized only allow height changes added by
+       addWindowSizeChanges */
+    if (w->state & CompWindowStateMaximizedVertMask)
+	xwcm &= ~CWHeight;
+
     xwcm |= addWindowSizeChanges (w, xwc,
 				  xwc->x, xwc->y,
 				  xwc->width, xwc->height,
@@ -3196,7 +3261,10 @@ moveResizeWindow (CompWindow     *w,
     if (w->mapNum && (xwcm & (CWWidth | CWHeight)))
 	sendSyncRequest (w);
 
-    XConfigureWindow (w->screen->display->display, w->id, xwcm, xwc);
+    configureXWindow (w, xwcm, xwc);
+
+    if (placed)
+	w->placed = TRUE;
 }
 
 void
@@ -3207,14 +3275,14 @@ updateWindowSize (CompWindow *w)
 
     mask = addWindowSizeChanges (w, &xwc,
 				 w->attrib.x, w->attrib.y,
-				 w->attrib.width, w->attrib.height,
-				 w->attrib.border_width);
+				 w->serverWidth, w->serverHeight,
+				 w->serverBorderWidth);
     if (mask)
     {
 	if (w->mapNum && (mask & (CWWidth | CWHeight)))
 	    sendSyncRequest (w);
 
-	configureXWindow (w->screen->display->display, w, mask, &xwc);
+	configureXWindow (w, mask, &xwc);
     }
 }
 
@@ -3271,8 +3339,7 @@ addWindowStackChanges (CompWindow     *w,
 
 	    for (; dw; dw = dw->prev)
 		if (dw->type & CompWindowTypeDockMask)
-		    configureXWindow (w->screen->display->display, dw,
-				      mask, xwc);
+		    configureXWindow (dw, mask, xwc);
 	}
     }
 
@@ -3287,7 +3354,7 @@ raiseWindow (CompWindow *w)
 
     mask = addWindowStackChanges (w, &xwc, findSiblingBelow (w, FALSE));
     if (mask)
-	configureXWindow (w->screen->display->display, w, mask, &xwc);
+	configureXWindow (w, mask, &xwc);
 }
 
 void
@@ -3298,7 +3365,7 @@ lowerWindow (CompWindow *w)
 
     mask = addWindowStackChanges (w, &xwc, findLowestSiblingBelow (w));
     if (mask)
-	configureXWindow (w->screen->display->display, w, mask, &xwc);
+	configureXWindow (w, mask, &xwc);
 }
 
 void
@@ -3316,7 +3383,7 @@ restackWindowAbove (CompWindow *w,
 
 	mask = addWindowStackChanges (w, &xwc, sibling);
 	if (mask)
-	    configureXWindow (w->screen->display->display, w, mask, &xwc);
+	    configureXWindow (w, mask, &xwc);
     }
 }
 
@@ -3335,7 +3402,7 @@ restackWindowBelow (CompWindow *w,
 
 	mask = addWindowStackChanges (w, &xwc, sibling);
 	if (mask)
-	    configureXWindow (w->screen->display->display, w, mask, &xwc);
+	    configureXWindow (w, mask, &xwc);
     }
 }
 
@@ -3361,8 +3428,8 @@ updateWindowAttributes (CompWindow *w,
     mask  = addWindowStackChanges (w, &xwc, findSiblingBelow (w, aboveFs));
     mask |= addWindowSizeChanges (w, &xwc,
 				  w->attrib.x, w->attrib.y,
-				  w->attrib.width, w->attrib.height,
-				  w->attrib.border_width);
+				  w->serverWidth, w->serverHeight,
+				  w->serverBorderWidth);
 
     if (!mask)
 	return;
@@ -3375,7 +3442,7 @@ updateWindowAttributes (CompWindow *w,
 	/* transient children above */
 	if (stackTransients (w, NULL, &xwc))
 	{
-	    configureXWindow (w->screen->display->display, w, mask, &xwc);
+	    configureXWindow (w, mask, &xwc);
 
 	    /* ancestors, sibilings and sibiling transients below */
 	    stackAncestors (w, &xwc);
@@ -3383,7 +3450,7 @@ updateWindowAttributes (CompWindow *w,
     }
     else
     {
-	configureXWindow (w->screen->display->display, w, mask, &xwc);
+	configureXWindow (w, mask, &xwc);
     }
 }
 
@@ -3403,9 +3470,9 @@ ensureWindowVisibility (CompWindow *w)
 	return;
 
     x1 = w->screen->workArea.x - w->screen->width * w->screen->x;
-    y1 = w->screen->workArea.y;
-    x2 = x1 + w->screen->workArea.width + w->screen->size * w->screen->width;
-    y2 = y1 + w->screen->workArea.height;
+    y1 = w->screen->workArea.y - w->screen->height * w->screen->y;
+    x2 = x1 + w->screen->workArea.width + w->screen->hsize * w->screen->width;
+    y2 = y1 + w->screen->workArea.height + w->screen->vsize * w->screen->height;
 
     if (w->attrib.x - w->input.left >= x2)
 	dx = (x2 - 25) - w->attrib.x;
@@ -3447,12 +3514,6 @@ activateWindow (CompWindow *w)
     moveInputFocusToWindow (w);
 }
 
-void
-activateWindowLight (CompWindow *w)
-{
-    moveInputFocusToWindow (w);
-}
-
 void
 closeWindow (CompWindow *w,
 	     Time	serverTime)
@@ -3515,6 +3576,7 @@ constrainNewWindowSize (CompWindow *w,
 			int        *newWidth,
 			int        *newHeight)
 {
+    CompDisplay      *d = w->screen->display;
     const XSizeHints *hints = &w->sizeHints;
     int		     min_width = 0;
     int		     min_height = 0;
@@ -3524,6 +3586,13 @@ constrainNewWindowSize (CompWindow *w,
     int		     yinc = 1;
     int		     max_width = MAXSHORT;
     int		     max_height = MAXSHORT;
+    long	     flags = hints->flags;
+
+    if (d->opt[COMP_DISPLAY_OPTION_IGNORE_HINTS_WHEN_MAXIMIZED].value.b)
+    {
+	if ((w->state & MAXIMIZE_STATE) == MAXIMIZE_STATE)
+	    flags &= ~(PResizeInc | PAspect);
+    }
 
     /* Ater gdk_window_constrain_size(), which is partially borrowed from fvwm.
      *
@@ -3538,21 +3607,21 @@ constrainNewWindowSize (CompWindow *w,
 #define FLOOR64(value, base) (((uint64_t) ((value) / (base))) * (base))
 #define CLAMP(v, min, max) ((v) <= (min) ? (min) : (v) >= (max) ? (max) : (v))
 
-    if ((hints->flags & PBaseSize) && (hints->flags & PMinSize))
+    if ((flags & PBaseSize) && (flags & PMinSize))
     {
 	base_width = hints->base_width;
 	base_height = hints->base_height;
 	min_width = hints->min_width;
 	min_height = hints->min_height;
     }
-    else if (hints->flags & PBaseSize)
+    else if (flags & PBaseSize)
     {
 	base_width = hints->base_width;
 	base_height = hints->base_height;
 	min_width = hints->base_width;
 	min_height = hints->base_height;
     }
-    else if (hints->flags & PMinSize)
+    else if (flags & PMinSize)
     {
 	base_width = hints->min_width;
 	base_height = hints->min_height;
@@ -3560,13 +3629,13 @@ constrainNewWindowSize (CompWindow *w,
 	min_height = hints->min_height;
     }
 
-    if (hints->flags & PMaxSize)
+    if (flags & PMaxSize)
     {
 	max_width = hints->max_width;
 	max_height = hints->max_height;
     }
 
-    if (hints->flags & PResizeInc)
+    if (flags & PResizeInc)
     {
 	xinc = MAX (xinc, hints->width_inc);
 	yinc = MAX (yinc, hints->height_inc);
@@ -3586,8 +3655,7 @@ constrainNewWindowSize (CompWindow *w,
      * ------------  <= -------- <=  -----------
      * min_aspect.y       height     max_aspect.y
      */
-    if (hints->flags & PAspect &&
-	hints->min_aspect.y > 0 && hints->max_aspect.x > 0)
+    if ((flags & PAspect) && hints->min_aspect.y > 0 && hints->max_aspect.x > 0)
     {
 	/* Use 64 bit arithmetic to prevent overflow */
 
@@ -3632,7 +3700,7 @@ constrainNewWindowSize (CompWindow *w,
 #undef FLOOR64
 #undef FLOOR
 
-    if (width != w->attrib.width || height != w->attrib.height)
+    if (width != w->serverWidth || height != w->serverHeight)
     {
 	*newWidth  = width;
 	*newHeight = height;
@@ -3682,6 +3750,8 @@ hideWindow (CompWindow *w)
 
     XUnmapWindow (w->screen->display->display, w->id);
 
+    (*w->screen->windowStateChangeNotify) (w);
+
     setWindowState (w->screen->display, w->state, w->id);
 }
 
@@ -3721,6 +3791,8 @@ showWindow (CompWindow *w)
 
     XMapWindow (w->screen->display->display, w->id);
 
+    (*w->screen->windowStateChangeNotify) (w);
+
     setWindowState (w->screen->display, w->state, w->id);
 }
 
@@ -3795,6 +3867,8 @@ maximizeWindow (CompWindow *w,
     recalcWindowType (w);
     recalcWindowActions (w);
 
+    (*w->screen->windowStateChangeNotify) (w);
+
     updateWindowAttributes (w, FALSE);
 
     setWindowState (w->screen->display, w->state, w->id);
@@ -3849,7 +3923,10 @@ focusWindowOnMap (CompWindow *w)
 		   CompWindowTypeDockMask))
 	return FALSE;
 
-    if (w->initialViewport != w->screen->x)
+    if (w->initialViewportX != w->screen->x)
+	return FALSE;
+
+    if (w->initialViewportY != w->screen->y)
 	return FALSE;
 
     if (!w->inputHint &&
@@ -3896,20 +3973,44 @@ redirectWindow (CompWindow *w)
    currently computed as the viewport where the center of the window is
    located, except for when the window is visible in the current viewport as
    the current viewport is then always returned. */
-int
-defaultViewportForWindow (CompWindow *w)
+void
+defaultViewportForWindow (CompWindow *w,
+			  int	     *vx,
+			  int	     *vy)
 {
     CompScreen *s = w->screen;
     int	       x;
+    int	       y;
 
-    if (w->attrib.x < s->width && w->attrib.x + w->width > 0)
-	return s->x;
+    if ((w->attrib.x < s->width && w->attrib.x + w->width > 0)	&&
+	(w->attrib.y < s->height && w->attrib.y + w->height > 0))
+    {
+	if (vx)
+	    *vx = s->x;
 
-    x = w->attrib.x + (w->width >> 1);
-    if (x < 0)
-	return s->x + ((x / s->width) - 1) % s->size;
-    else
-	return s->x + (x / s->width) % s->size;
+	if (vy)
+	    *vy = s->y;
+
+	return;
+    }
+
+    if (vx)
+    {
+	x = w->attrib.x + (w->width >> 1);
+	if (x < 0)
+	    *vx = s->x + ((x / s->width) - 1) % s->hsize;
+	else
+	    *vx = s->x + (x / s->width) % s->hsize;
+    }
+
+    if (vy)
+    {
+	y = w->attrib.y + (w->height >> 1);
+	if (y < 0)
+	    *vy = s->y + ((y / s->height) - 1) % s->vsize;
+	else
+	    *vy = s->y + (y / s->height) % s->vsize;
+    }
 }
 
 
@@ -4056,3 +4157,27 @@ freeWindowIcons (CompWindow *w)
 
     w->nIcon = 0;
 }
+
+int
+outputDeviceForWindow (CompWindow *w)
+{
+    int output = w->screen->currentOutputDev;
+    int x1, y1, x2, y2;
+
+    x1 = w->screen->outputDev[output].region.extents.x1;
+    y1 = w->screen->outputDev[output].region.extents.y1;
+    x2 = w->screen->outputDev[output].region.extents.x2;
+    y2 = w->screen->outputDev[output].region.extents.y2;
+
+    if (x1 > w->attrib.x + w->width  ||
+	y1 > w->attrib.y + w->height ||
+	x2 < w->attrib.x	     ||
+	y2 < w->attrib.y)
+    {
+	output = outputDeviceForPoint (w->screen,
+				       w->attrib.x + w->width  / 2,
+				       w->attrib.y + w->height / 2);
+    }
+
+    return output;
+}
